from .bases import *

class UserDetails(BaseEndpoint):

    def init(self):
        self.request_name = "user_details"
        self.request_param = {
            "user_id": None,
            "with_recently_downloaded": None
        }

class UserForgotPassword(BaseEndpoint):

    def init(self):
        self.request_name = "user_forgot_password"
        self.request_type = "post"
        self.request_param = {
            "email": None,
            "application_key": None
        }

class UserGetKey(BaseEndpoint):

    def init(self):
        self.request_name = "user_get_key"
        self.request_type = "post"
        self.request_param = {
            "username": None,
            "password": None,
            "application_key": None,
            "with_recently_downloaded": None
        }

class UserProfile(BaseEndpoint):

    def init(self):
        self.request_name = "user_profile"
        self.request_param = {
            "user_key": None
        }

class UserEditSettings(BaseEndpoint):

    def init(self):
        self.request_name = "user_edit_settings"
        self.request_type = "post"
        self.request_param = {
            "user_key": None,
            "application_key": None,
            "new_password": None,
            "about_text": None,
            "avatar_image": None
        }

class UserRegister(BaseEndpoint):

    def init(self):
        self.request_name = "user_register"
        self.request_type = "post"
        self.request_param = {
            "application_key": None,
            "username": None,
            "password": None,
            "email": None
        }

class UserResetPassword(BaseEndpoint):

    def init(self):
        self.request_name = "user_reset_password"
        self.request_type = "post"
        self.request_param = {
            "reset_code": None,
            "new_password": None,
            "application_key": None
        }

class MakeRequest(BaseEndpoint):

    def init(self):
        self.request_name = "make_request"
        self.request_type = "post"
        self.request_param = {
            "user_key": None,
            "movie_title": None,
            "request_message": None,
            "application_key": None
        }


YTS_API_URL = "http://yts.re/api/v2/"
YTS_DEFAULT_RESPONSE_FORMAT = "json"
YTS_STATUS_OK = "ok"
YTS_STATUS_ERROR = "error"

YTS_RESPONSE = {
    "status": YTS_STATUS_ERROR,
    "status_message": "",
    "data": {}
}


from .bases import *

class ListMovies(BaseEndpoint):

    def init(self):
        self.request_name = "list_movies"
        self.request_param = {
            "limit": None,
            "page": None,
            "quality": None,
            "minimum_rating": None,
            "query_term": None,
            "genre": None,
            "sort_by": None,
            "order_by": None,
            "with_rt_ratings": None
        }

class MovieDetails(BaseEndpoint):

    def init(self):
        self.request_name = "movie_details"
        self.request_param = {
            "movie_id": None,
            "with_images": None,
            "with_cast": None,
        }

class MovieSuggestions(BaseEndpoint):

    def init(self):
        self.request_name = "movie_suggestions"
        self.request_param = {
            "movie_id": None
        }

class MovieComments(BaseEndpoint):

    def init(self):
        self.request_name = "movie_comments"
        self.request_param = {
            "movie_id": None
        }

class MovieReviews(BaseEndpoint):

    def init(self):
        self.request_name = "movie_reviews"
        self.request_param = {
            "movie_id": None
        }

class MovieParentalGuides(BaseEndpoint):

    def init(self):
        self.request_name = "movie_parental_guides"
        self.request_param = {
            "movie_id": None
        }

class ListUpcoming(BaseEndpoint):

    def init(self):
        self.request_name = "list_upcoming"
        self.request_param = {}

class LikeMovie(BaseEndpoint):

    def init(self):
        self.request_name = "like_movie"
        self.request_type = "post"
        self.request_param = {
            "user_key": None,
            "movie_id": None,
            "application_key": None
        }

class GetMovieBookmarks(BaseEndpoint):

    def init(self):
        self.request_name = "get_movie_bookmarks"
        self.request_param = {
            "user_key": None,
            "with_rt_ratings": None
        }

class AddMovieBookmark(BaseEndpoint):

    def init(self):
        self.request_name = "add_movie_bookmark"
        self.request_type = "post"
        self.request_param = {
            "user_key": None,
            "movie_id": None,
            "application_key": None
        }

class DeleteMovieBookmark(BaseEndpoint):

    def init(self):
        self.request_name = "delete_movie_bookmark"
        self.request_type = "post"
        self.request_param = {
            "user_key": None,
            "movie_id": None,
            "application_key": None
        }

class MakeComment(BaseEndpoint):

    def init(self):
        self.request_name = "make_comment"
        self.request_type = "post"
        self.request_param = {
            "user_key": None,
            "movie_id": None,
            "comment_text": None,
            "application_key": None
        }

class LikeComment(BaseEndpoint):

    def init(self):
        self.request_name = "like_comment"
        self.request_type = "post"
        self.request_param = {
            "user_key": None,
            "comment_id": None,
            "application_key": None
        }

class ReportComment(BaseEndpoint):

    def init(self):
        self.request_name = "report_comment"
        self.request_type = "post"
        self.request_param = {
            "user_key": None,
            "comment_id": None,
            "application_key": None
        }

class DeleteComment(BaseEndpoint):

    def init(self):
        self.request_name = "delete_comment"
        self.request_type = "post"
        self.request_param = {
            "user_key": None,
            "comment_id": None,
            "application_key": None
        }


from .pyyts_client import *

__version__ = "1.1"
__app_name__ = "pyyts"
__description__ = "YTS API client"
__author__ = "Ferdinand Silva"
__author_email__ = "ferdinandsilva@ferdinandsilva.com"
__app_url__ = "http://ferdinandsilva.com"
__download_url__ = "https://github.com/six519/pyyts"


from .defaults import *
from .request_types import *

class YTSClient(object):

    def __init__(self, application_key=None):
        self.application_key = application_key
        self.movies = MoviesType(application_key)
        self.users = UsersType(application_key)


try:
    import requests
except:
    pass
    
from .defaults import *

class BaseEndpoint(object):

    def __init__(self, *args, **kwargs):
        self.request_param = kwargs.get("request_param", {})
        self.request_url = kwargs.get("request_url", YTS_API_URL)
        self.request_name = kwargs.get("request_name", "")
        self.request_type = kwargs.get("request_type", "get")
        self.request_response_format = kwargs.get("request_response_format", YTS_DEFAULT_RESPONSE_FORMAT)

        #OVERRIDE DEFAULT VALUES
        self.init()

    def init(self, **kwargs):
        raise NotImplementedError("Please override init")

    def __repr__(self):

        return self.request_name

    def request(self, **kwargs):

        ret = YTS_RESPONSE
        res = None

        if len(kwargs) > 0:
            self.request_param.update(kwargs)

        try:
            if self.request_type == "post":
                res = getattr(requests, self.request_type)("%s%s.%s" % (self.request_url, self.request_name, self.request_response_format), data=self.request_param)
            elif self.request_type == "get":
                res = getattr(requests, self.request_type)("%s%s.%s" % (self.request_url, self.request_name, self.request_response_format), params=self.request_param)
            else:
                raise Exception("Invalid request type!!")

            ret = res.json()
        except Exception as e:
            ret["status_message"] = str(e)

        return ret

    def listParams(self):
        print ", ".join(self.request_param.viewkeys())       

class BaseRequestType(object):

    def __init__(self, app_key=None):
        self.application_key = app_key
        self.type_name = ""

    def __repr__(self):
        return self.type_name


import pyyts

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(
    name=pyyts.__app_name__,
    version=pyyts.__version__,
    description=pyyts.__description__,
    author=pyyts.__author__,
    author_email=pyyts.__author_email__,
    packages=['pyyts'],
    install_requires=['requests'],
    url=pyyts.__app_url__,
    classifiers=(
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python',
        'License :: Freeware',
    ),
    download_url=pyyts.__download_url__,
)


from .bases import BaseRequestType
from .movies_endpoints import *
from .users_endpoints import *

class MoviesType(BaseRequestType):

    def __init__(self, app_key=None):
        BaseRequestType.__init__(self, app_key)
        self.type_name = "movies"

    def list_movies(self, **kwargs):
        return ListMovies().request(**kwargs)

    def movie_details(self, **kwargs):
        return MovieDetails().request(**kwargs)

    def movie_suggestions(self, **kwargs):
        return MovieSuggestions().request(**kwargs)

    def movie_comments(self, **kwargs):
        return MovieComments().request(**kwargs)

    def movie_reviews(self, **kwargs):
        return MovieReviews().request(**kwargs)

    def movie_parental_guides(self, **kwargs):
        return MovieParentalGuides().request(**kwargs)

    def list_upcoming(self, **kwargs):
        return ListUpcoming().request(**kwargs)

    def like_movie(self, **kwargs):
        kwargs["application_key"] = self.application_key
        return LikeMovie().request(**kwargs)

    def get_movie_bookmarks(self, **kwargs):
        return GetMovieBookmarks().request(**kwargs)

    def add_movie_bookmark(self, **kwargs):
        kwargs["application_key"] = self.application_key
        return AddMovieBookmark().request(**kwargs)

    def delete_movie_bookmark(self, **kwargs):
        kwargs["application_key"] = self.application_key
        return DeleteMovieBookmark().request(**kwargs)

    def make_comment(self, **kwargs):
        kwargs["application_key"] = self.application_key
        return MakeComment().request(**kwargs)

    def like_comment(self, **kwargs):
        kwargs["application_key"] = self.application_key
        return LikeComment().request(**kwargs)

    def report_comment(self, **kwargs):
        kwargs["application_key"] = self.application_key
        return ReportComment().request(**kwargs)

    def delete_comment(self, **kwargs):
        kwargs["application_key"] = self.application_key
        return DeleteComment().request(**kwargs)

class UsersType(BaseRequestType):

    def __init__(self, app_key=None):
        BaseRequestType.__init__(self, app_key)
        self.type_name = "users"

    def user_details(self, **kwargs):
        return UserDetails().request(**kwargs)

    def user_get_key(self, **kwargs):
        kwargs["application_key"] = self.application_key
        return UserGetKey().request(**kwargs)

    def user_forgot_password(self, **kwargs):
        kwargs["application_key"] = self.application_key
        return UserForgotPassword().request(**kwargs)

    def user_profile(self, **kwargs):
        return UserProfile().request(**kwargs)

    def user_edit_settings(self, **kwargs):
        kwargs["application_key"] = self.application_key
        return UserEditSettings().request(**kwargs)

    def user_register(self, **kwargs):
        kwargs["application_key"] = self.application_key
        return UserRegister().request(**kwargs)

    def user_reset_password(self, **kwargs):
        kwargs["application_key"] = self.application_key
        return UserResetPassword().request(**kwargs)

    def make_request(self, **kwargs):
        kwargs["application_key"] = self.application_key
        return MakeRequest().request(**kwargs)